(***********************************************************************)
(*                                                                     *)
(*                           Caml Light                                *)
(*                                                                     *)
(*            Pierre Weis, projet Cristal, INRIA Rocquencourt          *)
(*                                                                     *)
(*  Copyright 2000 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  Distributed only by permission.                   *)
(*                                                                     *)
(***********************************************************************)

#open "ppm";;
#open "pixmap";;
#open "graphics";;

(* Image manipulation functions *)

(* Build a new pixmap with pixels above threshold set to transp. *)
let make_transp threshold img =
 pixmap__map (fun c -> if c >= threshold then transp else c) img;;

(* Build a new pixmap with pixels almost white set to transp.
   (Conversely, only pixels that are ``dark enough'' are left in the
    resulting pixmap.) *)
(* graphics__white =        16777215 *)
let mk_transp = make_transp 16450000;;

(* Build a new pixmap with only dark pixels left.
   For images with clear bounds, render a ``wire effect'': only the
   shape of the drawing is left. *)
(* graphics__white =      16777215 *)
let mk_wire = make_transp 15000000;;

(* To animate the test. *)
let wait () =
 print_string "\nPress return to continue"; flush std_out;
 let _ = input_line stdin in
 ();;

let print_and_wait s =
 print_string s; print_newline (); wait ();;

(* To load and save in the proper directory. *)
let load_ppm_file s = load_ppm (filename__concat "ppms" s);;
let save_ppm_file s ppm = save_ppm (filename__concat "ppms" s) ppm;;
let save_bitmap_file s ppm = save_bitmap (filename__concat "ppms" s) ppm;;
let load_bitmap = load_ppm_file;;

(* Open the graphics window and clear it to black. *)
let open_black_graphics sz_x sz_y =
 open_graph (printf__sprintf " %dx%d" sz_x sz_y);
 set_color black; fill_rect 0 0 sz_x sz_y;;

open_black_graphics 600 450;;

(* Load and draw a reference image. *)
let fumi_pixmap = load_ppm_file "fumicaml.ppm";;
draw_pixmap fumi_pixmap 0 0;
print_string "FumiCaml loaded and drawn."; print_newline ();;

(* Build an image with transparencies out of the reference image and draw it. *)
let pixmap_transp = mk_transp fumi_pixmap;;
print_string "Transparent image built ..."; flush stdout;
draw_pixmap pixmap_transp 300 210;
print_and_wait " and drawn.";;

(* Save this transparent image, then reload it,
   and draw it again for easy visual comparison. *)
save_ppm_file "test_transp.ppm" pixmap_transp;
print_string "Transparent image saved ..."; flush stdout;;
let new_pixmap_transp = load_ppm_file "test_transp.ppm";;
print_string " loaded ..."; flush stdout;
draw_pixmap new_pixmap_transp 0 210;
print_and_wait " and drawn.";;

(* Create a bitmap pixmap out of the reference image. *)
let fumi_bitmap =
  pixmap__map (fun c -> if c >= 16400000 then white else black) fumi_pixmap;;

(* Save this bitmap image, then reload it,
   and draw it again for easy visual comparison. *)
save_bitmap_file "test_bitmap.ppm" fumi_bitmap;;
print_string "Bitmap image saved ..."; flush stdout;;
let bitmap = load_bitmap "test_bitmap.ppm";;
print_string " loaded ..."; flush stdout;
draw_pixmap bitmap 300 0;
print_and_wait " and drawn.";;

let pixmap_wire = mk_wire fumi_pixmap;;
 print_string "Drawing wire image..."; flush std_out;
 set_color black; fill_rect 300 0 256 162;
 draw_pixmap pixmap_wire 300 0;
 print_and_wait "Wire image drawn.";;

let show_wires start =
 for i = 0 to 10 do
  let range = white - start in
  let t = start + i * (range / 10) in
  print_string "Threshold is "; print_int t; print_newline ();
  let pixmap_wire = make_transp t fumi_pixmap in
  print_string "Drawing wire image..."; flush std_out;
  set_color black; fill_rect 300 0 256 162;
  draw_pixmap pixmap_wire 300 0;
  print_and_wait "Wire image drawn."
 done;;

let show_all_wires () =
 print_newline ();
 print_string "*******************************************"; print_newline ();
 print_string "Trying to set a threshold for wire imaging."; print_newline ();
 print_string "*******************************************"; print_newline ();
 print_and_wait "";

 print_string "*******************************************"; print_newline ();
 print_string "First step: coarse grain."; print_newline (); print_newline ();
 print_string "(Warning: first wire images may be invisible).";
 print_newline ();
 print_string "*******************************************"; print_newline ();
 print_and_wait "";
 show_wires 0;

 print_newline ();
 print_string "******************"; print_newline ();
 print_string "Second step: fine grain."; print_newline ();
 print_string "******************"; print_newline ();
 print_and_wait "";
 show_wires 13421768;

 print_newline ();
 print_string "******************"; print_newline ();
 print_string "Even finer grain."; print_newline ();
 print_string "******************"; print_newline ();
 print_and_wait "";
 show_wires 15521768;

 print_newline ();
 print_string "******************"; print_newline ();
 print_string "Finest grain."; print_newline ();
 print_string "******************"; print_newline ();
 print_and_wait "";
 show_wires 16400576;;

print_newline ();
print_string "*****************************************"; print_newline ();
print_string "Playing with transparency and background."; print_newline ();
print_string "(Always drawing the same pictures over\n";
print_string " various backgrounds.)"; print_newline ();
print_string "*****************************************"; print_newline ();
print_and_wait "";;

let f c =
 set_color c;
 fill_rect 0 210 256 162;
 fill_rect 300 0 256 162;

 draw_pixmap pixmap_transp 0 210;

 draw_pixmap pixmap_wire 300 0;

 wait ();;

list__do_list f [rgb 0 255 255; red; green; blue; yellow; cyan; black; white];;

show_all_wires ();;
